@@ -8,7 +8,6 @@ use std::sync::Arc;
 
 use fs_err as fs;
 use rustc_hash::FxHashSet;
-use tempfile::{tempdir, TempDir};
 use tracing::debug;
 
 pub use archive::ArchiveId;
@@ -121,7 +120,7 @@ pub struct Cache {
     ///
     /// Included to ensure that the temporary directory exists for the length of the operation, but
     /// is dropped at the end as appropriate.
-    _temp_dir_drop: Option<Arc<TempDir>>,
+    _temp_dir_drop: Option<Arc<tempfile::TempDir>>,
 }
 
 impl Cache {
@@ -136,7 +135,7 @@ impl Cache {
 
     /// Create a temporary cache directory.
     pub fn temp() -> Result<Self, io::Error> {
-        let temp_dir = tempdir()?;
+        let temp_dir = tempfile::tempdir()?;
         Ok(Self {
             root: temp_dir.path().to_path_buf(),
             refresh: Refresh::None(Timestamp::now()),
@@ -180,6 +179,12 @@ impl Cache {
         self.bucket(CacheBucket::Archive).join(id)
     }
 
+    /// Create an ephemeral Python environment in the cache.
+    pub fn environment(&self) -> io::Result<tempfile::TempDir> {
+        fs::create_dir_all(self.bucket(CacheBucket::Environments))?;
+        tempfile::tempdir_in(self.bucket(CacheBucket::Environments))
+    }
+
     /// Returns `true` if a cache entry must be revalidated given the [`Refresh`] policy.
     pub fn must_revalidate(&self, package: &PackageName) -> bool {
         match &self.refresh {
@@ -628,6 +633,8 @@ pub enum CacheBucket {
     /// that cache entries can be atomically replaced and removed, as storing directories in the
     /// other buckets directly would make atomic operations impossible.
     Archive,
+    /// Ephemeral virtual environments used to execute PEP 517 builds and other operations.
+    Environments,
 }
 
 impl CacheBucket {
@@ -637,9 +644,10 @@ impl CacheBucket {
             Self::FlatIndex => "flat-index-v0",
             Self::Git => "git-v0",
             Self::Interpreter => "interpreter-v2",
-            Self::Simple => "simple-v8",
+            Self::Simple => "simple-v9",
             Self::Wheels => "wheels-v1",
             Self::Archive => "archive-v0",
+            Self::Environments => "environments-v0",
         }
     }
 
@@ -750,20 +758,24 @@ impl CacheBucket {
             Self::Archive => {
                 // Nothing to do.
             }
+            Self::Environments => {
+                // Nothing to do.
+            }
         }
         Ok(summary)
     }
 
     /// Return an iterator over all cache buckets.
-    pub fn iter() -> impl Iterator<Item = CacheBucket> {
+    pub fn iter() -> impl Iterator<Item = Self> {
         [
-            CacheBucket::Wheels,
-            CacheBucket::BuiltWheels,
-            CacheBucket::FlatIndex,
-            CacheBucket::Git,
-            CacheBucket::Interpreter,
-            CacheBucket::Simple,
-            CacheBucket::Archive,
+            Self::Wheels,
+            Self::BuiltWheels,
+            Self::FlatIndex,
+            Self::Git,
+            Self::Interpreter,
+            Self::Simple,
+            Self::Archive,
+            Self::Environments,
         ]
         .iter()
         .copied()
