@@ -3,7 +3,7 @@ use std::path::PathBuf;
 use clap::Parser;
 use tracing::info;
 use uv_cache::{Cache, CacheArgs};
-use uv_toolchain::Toolchain;
+use uv_toolchain::{EnvironmentPreference, PythonEnvironment, ToolchainRequest};
 
 #[derive(Parser)]
 pub(crate) struct CompileArgs {
@@ -20,7 +20,12 @@ pub(crate) async fn compile(args: CompileArgs) -> anyhow::Result<()> {
     let interpreter = if let Some(python) = args.python {
         python
     } else {
-        let interpreter = Toolchain::find_virtualenv(&cache)?.into_interpreter();
+        let interpreter = PythonEnvironment::find(
+            &ToolchainRequest::default(),
+            EnvironmentPreference::OnlyVirtual,
+            &cache,
+        )?
+        .into_interpreter();
         interpreter.sys_executable().to_path_buf()
     };
 
