@@ -1,10 +1,9 @@
-use std::hash::BuildHasherDefault;
-
+use indexmap::IndexSet;
 use petgraph::{
     graph::{Graph, NodeIndex},
     Directed,
 };
-use rustc_hash::{FxHashMap, FxHashSet};
+use rustc_hash::{FxBuildHasher, FxHashMap};
 
 use distribution_types::{
     Dist, DistributionMetadata, Name, ResolutionDiagnostic, VersionId, VersionOrUrlRef,
@@ -56,10 +55,8 @@ impl ResolutionGraph {
     ) -> Result<Self, ResolveError> {
         let mut petgraph: Graph<AnnotatedDist, Option<MarkerTree>, Directed> =
             Graph::with_capacity(resolution.packages.len(), resolution.packages.len());
-        let mut inverse: FxHashMap<NodeKey, NodeIndex<u32>> = FxHashMap::with_capacity_and_hasher(
-            resolution.packages.len(),
-            BuildHasherDefault::default(),
-        );
+        let mut inverse: FxHashMap<NodeKey, NodeIndex<u32>> =
+            FxHashMap::with_capacity_and_hasher(resolution.packages.len(), FxBuildHasher);
         let mut diagnostics = Vec::new();
 
         // Add every package to the graph.
@@ -348,7 +345,7 @@ impl ResolutionGraph {
         }
 
         /// Add all marker parameters from the given tree to the given set.
-        fn add_marker_params_from_tree(marker_tree: &MarkerTree, set: &mut FxHashSet<MarkerParam>) {
+        fn add_marker_params_from_tree(marker_tree: &MarkerTree, set: &mut IndexSet<MarkerParam>) {
             match marker_tree {
                 MarkerTree::Expression(
                     MarkerExpression::Version { key, .. }
@@ -378,7 +375,7 @@ impl ResolutionGraph {
             }
         }
 
-        let mut seen_marker_values = FxHashSet::default();
+        let mut seen_marker_values = IndexSet::default();
         for i in self.petgraph.node_indices() {
             let dist = &self.petgraph[i];
             let version_id = match dist.version_or_url() {
