@@ -57,7 +57,7 @@ impl BatchPrefetcher {
             name,
             extra: None,
             dev: None,
-            marker: _marker,
+            marker: None,
             url: None,
         } = &**next
         else {
@@ -74,7 +74,7 @@ impl BatchPrefetcher {
         let versions_response = index
             .packages()
             .wait_blocking(name)
-            .ok_or(ResolveError::Unregistered)?;
+            .ok_or_else(|| ResolveError::UnregisteredTask(name.to_string()))?;
 
         let VersionsResponse::Found(ref version_map) = *versions_response else {
             return Ok(());
@@ -132,7 +132,8 @@ impl BatchPrefetcher {
             let CandidateDist::Compatible(dist) = candidate.dist() else {
                 continue;
             };
-            // Avoid building a lot of source distributions.
+
+            // Avoid prefetching source distributions, which could be expensive.
             if !dist.prefetchable() {
                 continue;
             }
@@ -166,11 +167,15 @@ impl BatchPrefetcher {
         // Only track base packages, no virtual packages from extras.
         if matches!(
             &*package,
-            PubGrubPackageInner::Package { extra: Some(_), .. }
+            PubGrubPackageInner::Package {
+                extra: None,
+                dev: None,
+                marker: None,
+                ..
+            }
         ) {
-            return;
+            *self.tried_versions.entry(package).or_default() += 1;
         }
-        *self.tried_versions.entry(package).or_default() += 1;
     }
 
     /// After 5, 10, 20, 40 tried versions, prefetch that many versions to start early but not
