@@ -3,13 +3,11 @@ use std::fmt::Write;
 use std::io::stdout;
 use std::ops::Deref;
 use std::path::Path;
-use std::str::FromStr;
 
 use anstream::{eprint, AutoStream, StripStream};
 use anyhow::{anyhow, Result};
 use itertools::Itertools;
 use owo_colors::OwoColorize;
-use pypi_types::Requirement;
 use tracing::debug;
 
 use distribution_types::{
@@ -17,19 +15,20 @@ use distribution_types::{
     Verbatim,
 };
 use install_wheel_rs::linker::LinkMode;
+use pypi_types::Requirement;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
     BuildOptions, Concurrency, ConfigSettings, Constraints, ExtrasSpecification, IndexStrategy,
-    Overrides, PreviewMode, SetupPyStrategy, Upgrade,
+    NoBinary, NoBuild, Overrides, PreviewMode, SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
 use uv_git::GitResolver;
-use uv_normalize::{ExtraName, PackageName};
+use uv_normalize::PackageName;
 use uv_requirements::{
     upgrade::read_requirements_txt, LookaheadResolver, NamedRequirementsResolver,
     RequirementsSource, RequirementsSpecification, SourceTreeResolver,
@@ -40,7 +39,8 @@ use uv_resolver::{
     Resolver,
 };
 use uv_toolchain::{
-    PythonEnvironment, PythonVersion, SystemPython, Toolchain, ToolchainRequest, VersionRequest,
+    EnvironmentPreference, PythonEnvironment, PythonVersion, Toolchain, ToolchainPreference,
+    ToolchainRequest, VersionRequest,
 };
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 use uv_warnings::warn_user;
@@ -71,6 +71,7 @@ pub(crate) async fn pip_compile(
     custom_compile_command: Option<String>,
     include_index_url: bool,
     include_find_links: bool,
+    include_build_options: bool,
     include_marker_expression: bool,
     include_index_annotation: bool,
     index_locations: IndexLocations,
@@ -88,6 +89,7 @@ pub(crate) async fn pip_compile(
     link_mode: LinkMode,
     python: Option<String>,
     system: bool,
+    toolchain_preference: ToolchainPreference,
     concurrency: Concurrency,
     native_tls: bool,
     quiet: bool,
@@ -153,14 +155,10 @@ pub(crate) async fn pip_compile(
     }
 
     // Find an interpreter to use for building distributions
-    let system = if system {
-        SystemPython::Required
-    } else {
-        SystemPython::Allowed
-    };
+    let environments = EnvironmentPreference::from_system_flag(system, false);
     let interpreter = if let Some(python) = python.as_ref() {
         let request = ToolchainRequest::parse(python);
-        Toolchain::find_requested(&request, system, preview, &cache)
+        Toolchain::find(&request, environments, toolchain_preference, &cache)
     } else {
         // TODO(zanieb): The split here hints at a problem with the abstraction; we should be able to use
         // `Toolchain::find(...)` here.
@@ -170,7 +168,7 @@ pub(crate) async fn pip_compile(
         } else {
             ToolchainRequest::default()
         };
-        Toolchain::find_best(&request, system, preview, &cache)
+        Toolchain::find_best(&request, environments, toolchain_preference, &cache)
     }?
     .into_interpreter();
 
@@ -284,15 +282,16 @@ pub(crate) async fn pip_compile(
         &source_index,
         &git,
         &in_flight,
+        index_strategy,
         setup_py,
         &config_settings,
         build_isolation,
         link_mode,
         &build_options,
+        exclude_newer,
         concurrency,
         preview,
-    )
-    .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
+    );
 
     // Resolve the requirements from the provided sources.
     let requirements = {
@@ -541,18 +540,17 @@ pub(crate) async fn pip_compile(
         writeln!(writer, "{}", format!("#    {relevant_markers}").green())?;
     }
 
-    // Write the index locations to the output channel.
-    let mut wrote_index = false;
+    let mut wrote_preamble = false;
 
     // If necessary, include the `--index-url` and `--extra-index-url` locations.
     if include_index_url {
         if let Some(index) = index_locations.index() {
             writeln!(writer, "--index-url {}", index.verbatim())?;
-            wrote_index = true;
+            wrote_preamble = true;
         }
         for extra_index in index_locations.extra_index() {
             writeln!(writer, "--extra-index-url {}", extra_index.verbatim())?;
-            wrote_index = true;
+            wrote_preamble = true;
         }
     }
 
@@ -560,12 +558,42 @@ pub(crate) async fn pip_compile(
     if include_find_links {
         for flat_index in index_locations.flat_index() {
             writeln!(writer, "--find-links {flat_index}")?;
-            wrote_index = true;
+            wrote_preamble = true;
+        }
+    }
+
+    // If necessary, include the `--no-binary` and `--only-binary` options.
+    if include_build_options {
+        match build_options.no_binary() {
+            NoBinary::None => {}
+            NoBinary::All => {
+                writeln!(writer, "--no-binary :all:")?;
+                wrote_preamble = true;
+            }
+            NoBinary::Packages(packages) => {
+                for package in packages {
+                    writeln!(writer, "--no-binary {package}")?;
+                    wrote_preamble = true;
+                }
+            }
+        }
+        match build_options.no_build() {
+            NoBuild::None => {}
+            NoBuild::All => {
+                writeln!(writer, "--only-binary :all:")?;
+                wrote_preamble = true;
+            }
+            NoBuild::Packages(packages) => {
+                for package in packages {
+                    writeln!(writer, "--only-binary {package}")?;
+                    wrote_preamble = true;
+                }
+            }
         }
     }
 
     // If we wrote an index, add a newline to separate it from the requirements
-    if wrote_index {
+    if wrote_preamble {
         writeln!(writer)?;
     }
 
@@ -723,15 +751,6 @@ impl OutputWriter {
     }
 }
 
-pub(crate) fn extra_name_with_clap_error(arg: &str) -> Result<ExtraName> {
-    ExtraName::from_str(arg).map_err(|_err| {
-        anyhow!(
-            "Extra names must start and end with a letter or digit and may only \
-            contain -, _, ., and alphanumeric characters"
-        )
-    })
-}
-
 /// An owned or unowned [`InMemoryIndex`].
 enum InMemoryIndexRef<'a> {
     Owned(InMemoryIndex),
