@@ -1,10 +1,12 @@
 use anyhow::Result;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_dispatch::BuildDispatch;
+use uv_distribution::pyproject::{Source, SourceError};
 use uv_distribution::pyproject_mut::PyProjectTomlMut;
 use uv_git::GitResolver;
 use uv_requirements::{NamedRequirementsResolver, RequirementsSource, RequirementsSpecification};
-use uv_resolver::{FlatIndex, InMemoryIndex, OptionsBuilder};
+use uv_resolver::{FlatIndex, InMemoryIndex};
+use uv_toolchain::{ToolchainPreference, ToolchainRequest};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
 
 use uv_cache::Cache;
@@ -20,12 +22,19 @@ use crate::printer::Printer;
 use crate::settings::ResolverInstallerSettings;
 
 /// Add one or more packages to the project requirements.
-#[allow(clippy::too_many_arguments)]
+#[allow(clippy::too_many_arguments, clippy::fn_params_excessive_bools)]
 pub(crate) async fn add(
     requirements: Vec<RequirementsSource>,
+    workspace: bool,
     dev: bool,
+    editable: Option<bool>,
+    raw: bool,
+    rev: Option<String>,
+    tag: Option<String>,
+    branch: Option<String>,
     python: Option<String>,
     settings: ResolverInstallerSettings,
+    toolchain_preference: ToolchainPreference,
     preview: PreviewMode,
     connectivity: Connectivity,
     concurrency: Concurrency,
@@ -41,7 +50,16 @@ pub(crate) async fn add(
     let project = ProjectWorkspace::discover(&std::env::current_dir()?, None).await?;
 
     // Discover or create the virtual environment.
-    let venv = project::init_environment(project.workspace(), python.as_deref(), cache, printer)?;
+    let venv = project::init_environment(
+        project.workspace(),
+        python.as_deref().map(ToolchainRequest::parse),
+        toolchain_preference,
+        connectivity,
+        native_tls,
+        cache,
+        printer,
+    )
+    .await?;
 
     let client_builder = BaseClientBuilder::new()
         .connectivity(connectivity)
@@ -97,18 +115,15 @@ pub(crate) async fn add(
         &index,
         &git,
         &in_flight,
+        settings.index_strategy,
         setup_py,
         &settings.config_setting,
         build_isolation,
         settings.link_mode,
         &settings.build_options,
+        settings.exclude_newer,
         concurrency,
         preview,
-    )
-    .with_options(
-        OptionsBuilder::new()
-            .exclude_newer(settings.exclude_newer)
-            .build(),
     );
 
     // Resolve any unnamed requirements.
@@ -124,11 +139,40 @@ pub(crate) async fn add(
 
     // Add the requirements to the `pyproject.toml`.
     let mut pyproject = PyProjectTomlMut::from_toml(project.current_project().pyproject_toml())?;
-    for req in requirements.into_iter().map(pep508_rs::Requirement::from) {
-        if dev {
-            pyproject.add_dev_dependency(&req)?;
+    for req in requirements {
+        let (req, source) = if raw {
+            // Use the PEP 508 requirement directly.
+            (pep508_rs::Requirement::from(req), None)
         } else {
-            pyproject.add_dependency(&req)?;
+            // Otherwise, try to construct the source.
+            let result = Source::from_requirement(
+                req.source.clone(),
+                workspace,
+                editable,
+                rev.clone(),
+                tag.clone(),
+                branch.clone(),
+            );
+
+            let source = match result {
+                Ok(source) => source,
+                Err(SourceError::UnresolvedReference(rev)) => {
+                    anyhow::bail!("Cannot resolve Git reference `{rev}` for requirement `{}`. Specify the reference with one of `--tag`, `--branch`, or `--rev`, or use the `--raw` flag.", req.name)
+                }
+                Err(err) => return Err(err.into()),
+            };
+
+            // Ignore the PEP 508 source.
+            let mut req = pep508_rs::Requirement::from(req);
+            req.clear_url();
+
+            (req, source)
+        };
+
+        if dev {
+            pyproject.add_dev_dependency(&req, source.as_ref())?;
+        } else {
+            pyproject.add_dependency(&req, source.as_ref())?;
         }
     }
 
@@ -144,13 +188,13 @@ pub(crate) async fn add(
         venv.interpreter(),
         &settings.upgrade,
         &settings.index_locations,
-        &settings.index_strategy,
-        &settings.keyring_provider,
-        &settings.resolution,
-        &settings.prerelease,
+        settings.index_strategy,
+        settings.keyring_provider,
+        settings.resolution,
+        settings.prerelease,
         &settings.config_setting,
-        settings.exclude_newer.as_ref(),
-        &settings.link_mode,
+        settings.exclude_newer,
+        settings.link_mode,
         &settings.build_options,
         preview,
         connectivity,
@@ -176,11 +220,11 @@ pub(crate) async fn add(
         Modifications::Sufficient,
         &settings.reinstall,
         &settings.index_locations,
-        &settings.index_strategy,
-        &settings.keyring_provider,
+        settings.index_strategy,
+        settings.keyring_provider,
         &settings.config_setting,
-        &settings.link_mode,
-        &settings.compile_bytecode,
+        settings.link_mode,
+        settings.compile_bytecode,
         &settings.build_options,
         preview,
         connectivity,
