@@ -7,12 +7,15 @@ use tokio::process::Command;
 use tracing::debug;
 
 use uv_cache::Cache;
+use uv_cli::ExternalCommand;
 use uv_client::{BaseClientBuilder, Connectivity};
 use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode};
 use uv_distribution::{ProjectWorkspace, Workspace};
 use uv_normalize::PackageName;
 use uv_requirements::RequirementsSource;
-use uv_toolchain::{PythonEnvironment, SystemPython, Toolchain, ToolchainRequest};
+use uv_toolchain::{
+    EnvironmentPreference, PythonEnvironment, Toolchain, ToolchainPreference, ToolchainRequest,
+};
 use uv_warnings::warn_user;
 
 use crate::commands::pip::operations::Modifications;
@@ -25,14 +28,14 @@ use crate::settings::ResolverInstallerSettings;
 pub(crate) async fn run(
     extras: ExtrasSpecification,
     dev: bool,
-    target: Option<String>,
-    mut args: Vec<OsString>,
+    command: ExternalCommand,
     requirements: Vec<RequirementsSource>,
     python: Option<String>,
     package: Option<PackageName>,
     settings: ResolverInstallerSettings,
     isolated: bool,
     preview: PreviewMode,
+    toolchain_preference: ToolchainPreference,
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
@@ -60,8 +63,16 @@ pub(crate) async fn run(
         } else {
             ProjectWorkspace::discover(&std::env::current_dir()?, None).await?
         };
-        let venv =
-            project::init_environment(project.workspace(), python.as_deref(), cache, printer)?;
+        let venv = project::init_environment(
+            project.workspace(),
+            python.as_deref().map(ToolchainRequest::parse),
+            toolchain_preference,
+            connectivity,
+            native_tls,
+            cache,
+            printer,
+        )
+        .await?;
 
         // Lock and sync the environment.
         let lock = project::lock::do_lock(
@@ -69,13 +80,13 @@ pub(crate) async fn run(
             venv.interpreter(),
             &settings.upgrade,
             &settings.index_locations,
-            &settings.index_strategy,
-            &settings.keyring_provider,
-            &settings.resolution,
-            &settings.prerelease,
+            settings.index_strategy,
+            settings.keyring_provider,
+            settings.resolution,
+            settings.prerelease,
             &settings.config_setting,
-            settings.exclude_newer.as_ref(),
-            &settings.link_mode,
+            settings.exclude_newer,
+            settings.link_mode,
             &settings.build_options,
             preview,
             connectivity,
@@ -95,11 +106,11 @@ pub(crate) async fn run(
             Modifications::Sufficient,
             &settings.reinstall,
             &settings.index_locations,
-            &settings.index_strategy,
-            &settings.keyring_provider,
+            settings.index_strategy,
+            settings.keyring_provider,
             &settings.config_setting,
-            &settings.link_mode,
-            &settings.compile_bytecode,
+            settings.link_mode,
+            settings.compile_bytecode,
             &settings.build_options,
             preview,
             connectivity,
@@ -131,8 +142,8 @@ pub(crate) async fn run(
             // Note we force preview on during `uv run` for now since the entire interface is in preview
             Toolchain::find_or_fetch(
                 python.as_deref().map(ToolchainRequest::parse),
-                SystemPython::Allowed,
-                PreviewMode::Enabled,
+                EnvironmentPreference::Any,
+                toolchain_preference,
                 client_builder,
                 cache,
             )
@@ -172,25 +183,25 @@ pub(crate) async fn run(
         )
     };
 
-    // Construct the command
-    let command = if let Some(target) = target {
+    let (target, args) = command.split();
+    let (command, prefix_args) = if let Some(target) = target {
         let target_path = PathBuf::from(&target);
         if target_path
             .extension()
             .map_or(false, |ext| ext.eq_ignore_ascii_case("py"))
             && target_path.exists()
         {
-            args.insert(0, target_path.as_os_str().into());
-            "python".to_string()
+            (OsString::from("python"), vec![target_path])
         } else {
-            target
+            (target.clone(), vec![])
         }
     } else {
-        "python".to_string()
+        (OsString::from("python"), vec![])
     };
 
     let mut process = Command::new(&command);
-    process.args(&args);
+    process.args(prefix_args);
+    process.args(args);
 
     // Construct the `PATH` environment variable.
     let new_path = std::env::join_paths(
@@ -243,12 +254,13 @@ pub(crate) async fn run(
     // TODO(zanieb): Throw a nicer error message if the command is not found
     let space = if args.is_empty() { "" } else { " " };
     debug!(
-        "Running `{command}{space}{}`",
+        "Running `{}{space}{}`",
+        command.to_string_lossy(),
         args.iter().map(|arg| arg.to_string_lossy()).join(" ")
     );
     let mut handle = process
         .spawn()
-        .with_context(|| format!("Failed to spawn: `{command}`"))?;
+        .with_context(|| format!("Failed to spawn: `{}`", command.to_string_lossy()))?;
     let status = handle.wait().await.context("Child process disappeared")?;
 
     // Exit based on the result of the command
