@@ -4,7 +4,7 @@ use std::fmt::Write;
 use uv_cache::Cache;
 use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
-use uv_toolchain::{SystemPython, Toolchain, ToolchainRequest};
+use uv_toolchain::{EnvironmentPreference, Toolchain, ToolchainPreference, ToolchainRequest};
 use uv_warnings::warn_user;
 
 use crate::commands::ExitStatus;
@@ -14,6 +14,7 @@ use crate::printer::Printer;
 #[allow(clippy::too_many_arguments)]
 pub(crate) async fn find(
     request: Option<String>,
+    toolchain_preference: ToolchainPreference,
     preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
@@ -26,10 +27,10 @@ pub(crate) async fn find(
         Some(request) => ToolchainRequest::parse(&request),
         None => ToolchainRequest::Any,
     };
-    let toolchain = Toolchain::find_requested(
+    let toolchain = Toolchain::find(
         &request,
-        SystemPython::Required,
-        PreviewMode::Enabled,
+        EnvironmentPreference::OnlySystem,
+        toolchain_preference,
         cache,
     )?;
 
