@@ -9,19 +9,24 @@ use miette::{Diagnostic, IntoDiagnostic};
 use owo_colors::OwoColorize;
 use thiserror::Error;
 
-use distribution_types::{IndexLocations, Requirement};
+use distribution_types::IndexLocations;
 use install_wheel_rs::linker::LinkMode;
+use pypi_types::Requirement;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
-use uv_client::{Connectivity, FlatIndexClient, RegistryClientBuilder};
-use uv_configuration::{Concurrency, KeyringProviderType, PreviewMode};
-use uv_configuration::{ConfigSettings, IndexStrategy, NoBinary, NoBuild, SetupPyStrategy};
+use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
+use uv_configuration::{
+    BuildOptions, Concurrency, ConfigSettings, IndexStrategy, KeyringProviderType, NoBinary,
+    NoBuild, PreviewMode, SetupPyStrategy,
+};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
-use uv_interpreter::{
-    find_default_interpreter, find_interpreter, InterpreterRequest, SourceSelector,
+use uv_git::GitResolver;
+use uv_resolver::{ExcludeNewer, FlatIndex, InMemoryIndex};
+use uv_toolchain::{
+    request_from_version_file, EnvironmentPreference, Toolchain, ToolchainPreference,
+    ToolchainRequest,
 };
-use uv_resolver::{ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder};
 use uv_types::{BuildContext, BuildIsolation, HashStrategy, InFlight};
 
 use crate::commands::{pip, ExitStatus};
@@ -37,6 +42,7 @@ use crate::shell::Shell;
 pub(crate) async fn venv(
     path: &Path,
     python_request: Option<&str>,
+    toolchain_preference: ToolchainPreference,
     link_mode: LinkMode,
     index_locations: &IndexLocations,
     index_strategy: IndexStrategy,
@@ -64,6 +70,7 @@ pub(crate) async fn venv(
         connectivity,
         seed,
         preview,
+        toolchain_preference,
         allow_existing,
         exclude_newer,
         native_tls,
@@ -113,22 +120,31 @@ async fn venv_impl(
     connectivity: Connectivity,
     seed: bool,
     preview: PreviewMode,
+    toolchain_preference: ToolchainPreference,
     allow_existing: bool,
     exclude_newer: Option<ExcludeNewer>,
     native_tls: bool,
     cache: &Cache,
     printer: Printer,
 ) -> miette::Result<ExitStatus> {
-    // Locate the Python interpreter.
-    let interpreter = if let Some(python) = python_request.as_ref() {
-        let system = uv_interpreter::SystemPython::Required;
-        let request = InterpreterRequest::parse(python);
-        let sources = SourceSelector::from_settings(system, preview);
-        find_interpreter(&request, system, &sources, cache)
-    } else {
-        find_default_interpreter(preview, cache)
+    let client_builder = BaseClientBuilder::default()
+        .connectivity(connectivity)
+        .native_tls(native_tls);
+
+    let mut interpreter_request = python_request.map(ToolchainRequest::parse);
+    if preview.is_enabled() && interpreter_request.is_none() {
+        interpreter_request = request_from_version_file().await.into_diagnostic()?;
     }
-    .into_diagnostic()?
+
+    // Locate the Python interpreter to use in the environment
+    let interpreter = Toolchain::find_or_fetch(
+        interpreter_request,
+        EnvironmentPreference::OnlySystem,
+        toolchain_preference,
+        client_builder,
+        cache,
+    )
+    .await
     .into_diagnostic()?
     .into_interpreter();
 
@@ -188,15 +204,15 @@ async fn venv_impl(
                 .map_err(VenvError::FlatIndex)?;
             FlatIndex::from_entries(
                 entries,
-                tags,
+                Some(tags),
                 &HashStrategy::None,
-                &NoBuild::All,
-                &NoBinary::None,
+                &BuildOptions::new(NoBinary::None, NoBuild::All),
             )
         };
 
         // Create a shared in-memory index.
         let index = InMemoryIndex::default();
+        let git = GitResolver::default();
 
         // Track in-flight downloads, builds, etc., across resolutions.
         let in_flight = InFlight::default();
@@ -205,6 +221,9 @@ async fn venv_impl(
         let config_settings = ConfigSettings::default();
         let concurrency = Concurrency::default();
 
+        // Do not allow builds
+        let build_options = BuildOptions::new(NoBinary::None, NoBuild::All);
+
         // Prep the build context.
         let build_dispatch = BuildDispatch::new(
             &client,
@@ -213,16 +232,18 @@ async fn venv_impl(
             index_locations,
             &flat_index,
             &index,
+            &git,
             &in_flight,
+            index_strategy,
             SetupPyStrategy::default(),
             &config_settings,
             BuildIsolation::Isolated,
             link_mode,
-            &NoBuild::All,
-            &NoBinary::None,
+            &build_options,
+            exclude_newer,
             concurrency,
-        )
-        .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
+            preview,
+        );
 
         // Resolve the seed packages.
         let requirements = if interpreter.python_tuple() < (3, 12) {
