@@ -1,5 +1,4 @@
 use std::env::VarError;
-use std::ffi::OsString;
 use std::num::NonZeroUsize;
 use std::path::PathBuf;
 use std::process;
@@ -10,6 +9,13 @@ use install_wheel_rs::linker::LinkMode;
 use pep508_rs::RequirementOrigin;
 use pypi_types::Requirement;
 use uv_cache::{CacheArgs, Refresh};
+use uv_cli::options::{flag, installer_options, resolver_installer_options, resolver_options};
+use uv_cli::{
+    AddArgs, ColorChoice, Commands, ExternalCommand, GlobalArgs, ListFormat, LockArgs, Maybe,
+    PipCheckArgs, PipCompileArgs, PipFreezeArgs, PipInstallArgs, PipListArgs, PipShowArgs,
+    PipSyncArgs, PipTreeArgs, PipUninstallArgs, RemoveArgs, RunArgs, SyncArgs, ToolRunArgs,
+    ToolchainFindArgs, ToolchainInstallArgs, ToolchainListArgs, VenvArgs,
+};
 use uv_client::Connectivity;
 use uv_configuration::{
     BuildOptions, Concurrency, ConfigSettings, ExtrasSpecification, IndexStrategy,
@@ -17,22 +23,15 @@ use uv_configuration::{
     Upgrade,
 };
 use uv_normalize::PackageName;
+use uv_requirements::RequirementsSource;
 use uv_resolver::{AnnotationStyle, DependencyMode, ExcludeNewer, PreReleaseMode, ResolutionMode};
 use uv_settings::{
     Combine, FilesystemOptions, InstallerOptions, Options, PipOptions, ResolverInstallerOptions,
     ResolverOptions,
 };
-use uv_toolchain::{Prefix, PythonVersion, Target};
+use uv_toolchain::{Prefix, PythonVersion, Target, ToolchainPreference};
 
-use crate::cli::{
-    AddArgs, BuildArgs, ColorChoice, GlobalArgs, IndexArgs, InstallerArgs, LockArgs, Maybe,
-    PipCheckArgs, PipCompileArgs, PipFreezeArgs, PipInstallArgs, PipListArgs, PipShowArgs,
-    PipSyncArgs, PipUninstallArgs, RefreshArgs, RemoveArgs, ResolverArgs, ResolverInstallerArgs,
-    RunArgs, SyncArgs, ToolRunArgs, ToolchainFindArgs, ToolchainInstallArgs, ToolchainListArgs,
-    VenvArgs,
-};
 use crate::commands::pip::operations::Modifications;
-use crate::commands::ListFormat;
 
 /// The resolved global settings to use for any invocation of the CLI.
 #[allow(clippy::struct_excessive_bools)]
@@ -46,11 +45,35 @@ pub(crate) struct GlobalSettings {
     pub(crate) isolated: bool,
     pub(crate) show_settings: bool,
     pub(crate) preview: PreviewMode,
+    pub(crate) toolchain_preference: ToolchainPreference,
 }
 
 impl GlobalSettings {
     /// Resolve the [`GlobalSettings`] from the CLI and filesystem configuration.
-    pub(crate) fn resolve(args: &GlobalArgs, workspace: Option<&FilesystemOptions>) -> Self {
+    pub(crate) fn resolve(
+        command: &Commands,
+        args: &GlobalArgs,
+        workspace: Option<&FilesystemOptions>,
+    ) -> Self {
+        let preview = PreviewMode::from(
+            flag(args.preview, args.no_preview)
+                .combine(workspace.and_then(|workspace| workspace.globals.preview))
+                .unwrap_or(false),
+        );
+
+        // Always use preview mode toolchain preferences during preview commands
+        // TODO(zanieb): There should be a cleaner way to do this, we should probably resolve
+        // force preview to true for these commands but it would break our experimental warning
+        // right now
+        let default_toolchain_preference = if matches!(
+            command,
+            Commands::Project(_) | Commands::Toolchain(_) | Commands::Tool(_)
+        ) {
+            ToolchainPreference::default_from(PreviewMode::Enabled)
+        } else {
+            ToolchainPreference::default_from(preview)
+        };
+
         Self {
             quiet: args.quiet,
             verbose: args.verbose,
@@ -84,11 +107,11 @@ impl GlobalSettings {
             },
             isolated: args.isolated,
             show_settings: args.show_settings,
-            preview: PreviewMode::from(
-                flag(args.preview, args.no_preview)
-                    .combine(workspace.and_then(|workspace| workspace.globals.preview))
-                    .unwrap_or(false),
-            ),
+            preview,
+            toolchain_preference: args
+                .toolchain_preference
+                .combine(workspace.and_then(|workspace| workspace.globals.toolchain_preference))
+                .unwrap_or(default_toolchain_preference),
         }
     }
 }
@@ -122,8 +145,7 @@ impl CacheSettings {
 pub(crate) struct RunSettings {
     pub(crate) extras: ExtrasSpecification,
     pub(crate) dev: bool,
-    pub(crate) target: Option<String>,
-    pub(crate) args: Vec<OsString>,
+    pub(crate) command: ExternalCommand,
     pub(crate) with: Vec<String>,
     pub(crate) python: Option<String>,
     pub(crate) package: Option<PackageName>,
@@ -141,8 +163,7 @@ impl RunSettings {
             no_all_extras,
             dev,
             no_dev,
-            target,
-            args,
+            command,
             with,
             installer,
             build,
@@ -157,8 +178,7 @@ impl RunSettings {
                 extra.unwrap_or_default(),
             ),
             dev: flag(dev, no_dev).unwrap_or(true),
-            target,
-            args,
+            command,
             with,
             python,
             package,
@@ -175,8 +195,7 @@ impl RunSettings {
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
 pub(crate) struct ToolRunSettings {
-    pub(crate) target: String,
-    pub(crate) args: Vec<OsString>,
+    pub(crate) command: ExternalCommand,
     pub(crate) from: Option<String>,
     pub(crate) with: Vec<String>,
     pub(crate) python: Option<String>,
@@ -189,8 +208,7 @@ impl ToolRunSettings {
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(args: ToolRunArgs, filesystem: Option<FilesystemOptions>) -> Self {
         let ToolRunArgs {
-            target,
-            args,
+            command,
             from,
             with,
             installer,
@@ -200,8 +218,7 @@ impl ToolRunSettings {
         } = args;
 
         Self {
-            target,
-            args,
+            command,
             from,
             with,
             python,
@@ -373,8 +390,14 @@ impl LockSettings {
 #[allow(clippy::struct_excessive_bools, dead_code)]
 #[derive(Debug, Clone)]
 pub(crate) struct AddSettings {
-    pub(crate) requirements: Vec<String>,
+    pub(crate) requirements: Vec<RequirementsSource>,
     pub(crate) dev: bool,
+    pub(crate) workspace: bool,
+    pub(crate) editable: Option<bool>,
+    pub(crate) raw: bool,
+    pub(crate) rev: Option<String>,
+    pub(crate) tag: Option<String>,
+    pub(crate) branch: Option<String>,
     pub(crate) python: Option<String>,
     pub(crate) refresh: Refresh,
     pub(crate) settings: ResolverInstallerSettings,
@@ -387,15 +410,32 @@ impl AddSettings {
         let AddArgs {
             requirements,
             dev,
+            workspace,
+            editable,
+            raw,
+            rev,
+            tag,
+            branch,
             installer,
             build,
             refresh,
             python,
         } = args;
 
+        let requirements = requirements
+            .into_iter()
+            .map(RequirementsSource::Package)
+            .collect::<Vec<_>>();
+
         Self {
             requirements,
+            workspace,
             dev,
+            editable,
+            raw,
+            rev,
+            tag,
+            branch,
             python,
             refresh: Refresh::from(refresh),
             settings: ResolverInstallerSettings::combine(
@@ -488,6 +528,8 @@ impl PipCompileSettings {
             no_emit_index_url,
             emit_find_links,
             no_emit_find_links,
+            emit_build_options,
+            no_emit_build_options,
             emit_marker_expression,
             no_emit_marker_expression,
             emit_index_annotation,
@@ -541,6 +583,7 @@ impl PipCompileSettings {
                     no_emit_package,
                     emit_index_url: flag(emit_index_url, no_emit_index_url),
                     emit_find_links: flag(emit_find_links, no_emit_find_links),
+                    emit_build_options: flag(emit_build_options, no_emit_build_options),
                     emit_marker_expression: flag(emit_marker_expression, no_emit_marker_expression),
                     emit_index_annotation: flag(emit_index_annotation, no_emit_index_annotation),
                     annotation_style,
@@ -907,6 +950,40 @@ impl PipShowSettings {
     }
 }
 
+/// The resolved settings to use for a `pip show` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct PipTreeSettings {
+    // CLI-only settings.
+    pub(crate) shared: PipSettings,
+}
+
+impl PipTreeSettings {
+    /// Resolve the [`PipTreeSettings`] from the CLI and workspace configuration.
+    pub(crate) fn resolve(args: PipTreeArgs, filesystem: Option<FilesystemOptions>) -> Self {
+        let PipTreeArgs {
+            strict,
+            no_strict,
+            python,
+            system,
+            no_system,
+        } = args;
+
+        Self {
+            // Shared settings.
+            shared: PipSettings::combine(
+                PipOptions {
+                    python,
+                    system: flag(system, no_system),
+                    strict: flag(strict, no_strict),
+                    ..PipOptions::default()
+                },
+                filesystem,
+            ),
+        }
+    }
+}
+
 /// The resolved settings to use for a `pip check` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
@@ -1332,6 +1409,7 @@ pub(crate) struct PipSettings {
     pub(crate) no_emit_package: Vec<PackageName>,
     pub(crate) emit_index_url: bool,
     pub(crate) emit_find_links: bool,
+    pub(crate) emit_build_options: bool,
     pub(crate) emit_marker_expression: bool,
     pub(crate) emit_index_annotation: bool,
     pub(crate) annotation_style: AnnotationStyle,
@@ -1386,6 +1464,7 @@ impl PipSettings {
             no_emit_package,
             emit_index_url,
             emit_find_links,
+            emit_build_options,
             emit_marker_expression,
             emit_index_annotation,
             annotation_style,
@@ -1521,6 +1600,10 @@ impl PipSettings {
                 .emit_find_links
                 .combine(emit_find_links)
                 .unwrap_or_default(),
+            emit_build_options: args
+                .emit_build_options
+                .combine(emit_build_options)
+                .unwrap_or_default(),
             emit_marker_expression: args
                 .emit_marker_expression
                 .combine(emit_marker_expression)
@@ -1632,346 +1715,3 @@ fn parse_failure(name: &str, expected: &str) -> ! {
     eprintln!("error: invalid value for {name}, expected {expected}");
     process::exit(1)
 }
-
-/// Given a boolean flag pair (like `--upgrade` and `--no-upgrade`), resolve the value of the flag.
-fn flag(yes: bool, no: bool) -> Option<bool> {
-    match (yes, no) {
-        (true, false) => Some(true),
-        (false, true) => Some(false),
-        (false, false) => None,
-        (..) => unreachable!("Clap should make this impossible"),
-    }
-}
-
-impl From<RefreshArgs> for Refresh {
-    fn from(value: RefreshArgs) -> Self {
-        let RefreshArgs {
-            refresh,
-            no_refresh,
-            refresh_package,
-        } = value;
-
-        Self::from_args(flag(refresh, no_refresh), refresh_package)
-    }
-}
-
-impl From<ResolverArgs> for PipOptions {
-    fn from(args: ResolverArgs) -> Self {
-        let ResolverArgs {
-            index_args,
-            upgrade,
-            no_upgrade,
-            upgrade_package,
-            index_strategy,
-            keyring_provider,
-            resolution,
-            prerelease,
-            pre,
-            config_setting,
-            exclude_newer,
-            link_mode,
-        } = args;
-
-        Self {
-            upgrade: flag(upgrade, no_upgrade),
-            upgrade_package: Some(upgrade_package),
-            index_strategy,
-            keyring_provider,
-            resolution,
-            prerelease: if pre {
-                Some(PreReleaseMode::Allow)
-            } else {
-                prerelease
-            },
-            config_settings: config_setting
-                .map(|config_settings| config_settings.into_iter().collect::<ConfigSettings>()),
-            exclude_newer,
-            link_mode,
-            ..PipOptions::from(index_args)
-        }
-    }
-}
-
-impl From<InstallerArgs> for PipOptions {
-    fn from(args: InstallerArgs) -> Self {
-        let InstallerArgs {
-            index_args,
-            reinstall,
-            no_reinstall,
-            reinstall_package,
-            index_strategy,
-            keyring_provider,
-            config_setting,
-            link_mode,
-            compile_bytecode,
-            no_compile_bytecode,
-        } = args;
-
-        Self {
-            reinstall: flag(reinstall, no_reinstall),
-            reinstall_package: Some(reinstall_package),
-            index_strategy,
-            keyring_provider,
-            config_settings: config_setting
-                .map(|config_settings| config_settings.into_iter().collect::<ConfigSettings>()),
-            link_mode,
-            compile_bytecode: flag(compile_bytecode, no_compile_bytecode),
-            ..PipOptions::from(index_args)
-        }
-    }
-}
-
-impl From<ResolverInstallerArgs> for PipOptions {
-    fn from(args: ResolverInstallerArgs) -> Self {
-        let ResolverInstallerArgs {
-            index_args,
-            upgrade,
-            no_upgrade,
-            upgrade_package,
-            reinstall,
-            no_reinstall,
-            reinstall_package,
-            index_strategy,
-            keyring_provider,
-            resolution,
-            prerelease,
-            pre,
-            config_setting,
-            exclude_newer,
-            link_mode,
-            compile_bytecode,
-            no_compile_bytecode,
-        } = args;
-
-        Self {
-            upgrade: flag(upgrade, no_upgrade),
-            upgrade_package: Some(upgrade_package),
-            reinstall: flag(reinstall, no_reinstall),
-            reinstall_package: Some(reinstall_package),
-            index_strategy,
-            keyring_provider,
-            resolution,
-            prerelease: if pre {
-                Some(PreReleaseMode::Allow)
-            } else {
-                prerelease
-            },
-            config_settings: config_setting
-                .map(|config_settings| config_settings.into_iter().collect::<ConfigSettings>()),
-            exclude_newer,
-            link_mode,
-            compile_bytecode: flag(compile_bytecode, no_compile_bytecode),
-            ..PipOptions::from(index_args)
-        }
-    }
-}
-
-impl From<IndexArgs> for PipOptions {
-    fn from(args: IndexArgs) -> Self {
-        let IndexArgs {
-            index_url,
-            extra_index_url,
-            no_index,
-            find_links,
-        } = args;
-
-        Self {
-            index_url: index_url.and_then(Maybe::into_option),
-            extra_index_url: extra_index_url.map(|extra_index_urls| {
-                extra_index_urls
-                    .into_iter()
-                    .filter_map(Maybe::into_option)
-                    .collect()
-            }),
-            no_index: if no_index { Some(true) } else { None },
-            find_links,
-            ..PipOptions::default()
-        }
-    }
-}
-
-/// Construct the [`InstallerOptions`] from the [`InstallerArgs`] and [`BuildArgs`].
-fn installer_options(installer_args: InstallerArgs, build_args: BuildArgs) -> InstallerOptions {
-    let InstallerArgs {
-        index_args,
-        reinstall,
-        no_reinstall,
-        reinstall_package,
-        index_strategy,
-        keyring_provider,
-        config_setting,
-        link_mode,
-        compile_bytecode,
-        no_compile_bytecode,
-    } = installer_args;
-
-    let BuildArgs {
-        no_build,
-        build,
-        no_build_package,
-        no_binary,
-        binary,
-        no_binary_package,
-    } = build_args;
-
-    InstallerOptions {
-        index_url: index_args.index_url.and_then(Maybe::into_option),
-        extra_index_url: index_args.extra_index_url.map(|extra_index_urls| {
-            extra_index_urls
-                .into_iter()
-                .filter_map(Maybe::into_option)
-                .collect()
-        }),
-        no_index: if index_args.no_index {
-            Some(true)
-        } else {
-            None
-        },
-        find_links: index_args.find_links,
-        reinstall: flag(reinstall, no_reinstall),
-        reinstall_package: Some(reinstall_package),
-        index_strategy,
-        keyring_provider,
-        config_settings: config_setting
-            .map(|config_settings| config_settings.into_iter().collect::<ConfigSettings>()),
-        link_mode,
-        compile_bytecode: flag(compile_bytecode, no_compile_bytecode),
-        no_build: flag(no_build, build),
-        no_build_package: Some(no_build_package),
-        no_binary: flag(no_binary, binary),
-        no_binary_package: Some(no_binary_package),
-    }
-}
-
-/// Construct the [`ResolverOptions`] from the [`ResolverArgs`] and [`BuildArgs`].
-fn resolver_options(resolver_args: ResolverArgs, build_args: BuildArgs) -> ResolverOptions {
-    let ResolverArgs {
-        index_args,
-        upgrade,
-        no_upgrade,
-        upgrade_package,
-        index_strategy,
-        keyring_provider,
-        resolution,
-        prerelease,
-        pre,
-        config_setting,
-        exclude_newer,
-        link_mode,
-    } = resolver_args;
-
-    let BuildArgs {
-        no_build,
-        build,
-        no_build_package,
-        no_binary,
-        binary,
-        no_binary_package,
-    } = build_args;
-
-    ResolverOptions {
-        index_url: index_args.index_url.and_then(Maybe::into_option),
-        extra_index_url: index_args.extra_index_url.map(|extra_index_urls| {
-            extra_index_urls
-                .into_iter()
-                .filter_map(Maybe::into_option)
-                .collect()
-        }),
-        no_index: if index_args.no_index {
-            Some(true)
-        } else {
-            None
-        },
-        find_links: index_args.find_links,
-        upgrade: flag(upgrade, no_upgrade),
-        upgrade_package: Some(upgrade_package),
-        index_strategy,
-        keyring_provider,
-        resolution,
-        prerelease: if pre {
-            Some(PreReleaseMode::Allow)
-        } else {
-            prerelease
-        },
-        config_settings: config_setting
-            .map(|config_settings| config_settings.into_iter().collect::<ConfigSettings>()),
-        exclude_newer,
-        link_mode,
-        no_build: flag(no_build, build),
-        no_build_package: Some(no_build_package),
-        no_binary: flag(no_binary, binary),
-        no_binary_package: Some(no_binary_package),
-    }
-}
-
-/// Construct the [`ResolverInstallerOptions`] from the [`ResolverInstallerArgs`] and [`BuildArgs`].
-fn resolver_installer_options(
-    resolver_installer_args: ResolverInstallerArgs,
-    build_args: BuildArgs,
-) -> ResolverInstallerOptions {
-    let ResolverInstallerArgs {
-        index_args,
-        upgrade,
-        no_upgrade,
-        upgrade_package,
-        reinstall,
-        no_reinstall,
-        reinstall_package,
-        index_strategy,
-        keyring_provider,
-        resolution,
-        prerelease,
-        pre,
-        config_setting,
-        exclude_newer,
-        link_mode,
-        compile_bytecode,
-        no_compile_bytecode,
-    } = resolver_installer_args;
-
-    let BuildArgs {
-        no_build,
-        build,
-        no_build_package,
-        no_binary,
-        binary,
-        no_binary_package,
-    } = build_args;
-
-    ResolverInstallerOptions {
-        index_url: index_args.index_url.and_then(Maybe::into_option),
-        extra_index_url: index_args.extra_index_url.map(|extra_index_urls| {
-            extra_index_urls
-                .into_iter()
-                .filter_map(Maybe::into_option)
-                .collect()
-        }),
-        no_index: if index_args.no_index {
-            Some(true)
-        } else {
-            None
-        },
-        find_links: index_args.find_links,
-        upgrade: flag(upgrade, no_upgrade),
-        upgrade_package: Some(upgrade_package),
-        reinstall: flag(reinstall, no_reinstall),
-        reinstall_package: Some(reinstall_package),
-        index_strategy,
-        keyring_provider,
-        resolution,
-        prerelease: if pre {
-            Some(PreReleaseMode::Allow)
-        } else {
-            prerelease
-        },
-        config_settings: config_setting
-            .map(|config_settings| config_settings.into_iter().collect::<ConfigSettings>()),
-        exclude_newer,
-        link_mode,
-        compile_bytecode: flag(compile_bytecode, no_compile_bytecode),
-        no_build: flag(no_build, build),
-        no_build_package: Some(no_build_package),
-        no_binary: flag(no_binary, binary),
-        no_binary_package: Some(no_binary_package),
-    }
-}
